#!/usr/bin/env python

import os;
import dircache;
import sys;
import urllib

if __name__ == "__main__":
    os.chdir(sys.argv[1])
else:
    from wvtest import *

class DimValue:
    def __init__(self, dim, value):
        self.dim = dim
        self.value = value
    def __repr__(self):
        return repr(self.value)
    def htmlLabel(self):
        return self.dim.htmlLabel(self.value)

class Dimension(dict):
    def __init__(self, atype, name=None):
        self.type = atype
        if (name):
            self.name = name
        else:
            self.name = atype            

    def __iter__(self):
        keys = self.keys()
        keys.sort()
        for k in keys:
            yield self[k]

    def addValue(self, *values):
        for value in values:
            if value not in self:
                self[value] = DimValue(self, value)
    def htmlLabel(self, v):
        return v

@wvtest
def Dimension_and_DimValue():
    d = Dimension('kern', 'Kernel')
    d.addValue('2.6.31')
    d.addValue('2.6.30')
    l=[]
    for v in d:
        WVPASSEQ(v.__class__, DimValue)
        l.append(v.value)
    lsorted = ['2.6.30', '2.6.31']
    WVPASSEQ(l, lsorted)
    WVPASSEQ(v.htmlLabel(), '2.6.31')
    WVPASSEQ(d['2.6.31'].value, '2.6.31')
    

class DimensionKern(Dimension):
    def __init__(self):
        Dimension.__init__(self, 'kern', 'Kernel')
    def htmlLabel(self, v):
        i=v.find(":")
        if i>0: kver=v[:i]
        else: kver=v
        return v+"<br><a href='config-%s'>config</a>"%(urllib.quote(kver))
    def versions(self):
        for v in self.values:
            i=v.find(":")
            if i>0: kver=v[:i]
            else: kver=v
            yield kver
        
class DimensionTest(Dimension):
    def __init__(self):
        Dimension.__init__(self, 'test', 'Test')
    def htmlLabel(self, v):
        return v+"<br><a href='%s.sh.html'>source</a>"%(urllib.quote(v))

class Test:
    @classmethod
    def isOnPath(cls, path):
        f = os.path.join(path, 'plot.gp')
        return os.path.isfile(f)
    def __init__(self, path):
        self.path = path

def iterDimValues(dimensions):
    idx = [0 for i in xrange(len(dimensions))]
    done=False
    while not done:
        values=[]
        for i in xrange(len(dimensions)):
            values.append(dimensions[i].values()[idx[i]])
        yield values
        done=True
        for i in xrange(len(dimensions)):
            idx[i] += 1
            if idx[i] < len(dimensions[i]):
                done=False
                break
            idx[i] = 0

class Tests(dict):
    """Represents all tests organized along several dimensions"""
    def __init__(self, rootpath, *dimensions):
        dict.__init__(self)
        self.dimensions = dimensions
        if (rootpath):
            self.populate(rootpath)
    def getTest(self, key):
        realkey=[]
        for d in self.dimensions:
            for i in key:
                if i.dim == d:
                    realkey.append(i.value)
        if len(realkey) != len(self.dimensions):
            raise KeyError("The coordinates in key do not match dimensions")
        return self[tuple(realkey)]
        
    def addTest(self, test, coordinates):
        if len(coordinates) != len(self.dimensions):
            raise KeyError("The number coordinates do not match the number of dimensions")
        self[tuple(coordinates)] = test
        for i in xrange(len(coordinates)):
            self.dimensions[i].addValue(coordinates[i])

    def populate(self, rootpath):
        for root, dirs, files in os.walk(rootpath):
            if (root.find(rootpath) == 0):
                coordinates = root[len(rootpath):]
            else:
                coordinates = rootpath
            if Test.isOnPath(root):
                self.addTest(Test(root), coordinates.split("/"))
    def iterDimensionPairs(self):
        for i in xrange(len(self.dimensions)):
            for j in xrange(i+1, len(self.dimensions)):
                yield (self.dimensions[i], self.dimensions[j])
                yield (self.dimensions[j], self.dimensions[i])
    def iterRemainingDimensions(self, dimensionPair):
        for d in self.dimensions:
            if d not in dimensionPair:
                yield d
    def generateHtml(self):
        for pair in self.iterDimensionPairs():
            remdims = self.iterRemainingDimensions(pair)
            for vals in iterDimValues(remdims):
                self.generatePage(pair, remdims, vals)

    def generatePage(self, dimpair, dimother, valsother):
        dimy, dimx = dimpair
        html = open("%s-vs-%s-%s.html"%(dimy.type, dimx.type, "-".join([v.value for v in valsother])), "w")
        title = "CAN gateway benchmark for " + ", ".join([v.dim.name+" "+v.value for v in valsother])
        print >> html, """<html>
<head>
<title>%s</title>
<style>
img { border: 0; }
table { border-collapse: collapse; }
th, td { border: 1px solid lightgray; padding: 4px;}
</style>
</head>
<body>
<h1>%s</h1>"""  % (title, title)
        for d in dimother:
            pass
#             print >>html, "View for %s: " % str(ps.pageclass.name)
#             for v in ps.values:
#                 print >>html, "<a href='%s-%s.html'>%s</a> | "%(ps.values.type, urllib.quote(v), v)
#             print >>html, "<br>"
#             try:
#                 print >>html, d.htmlPreamble()
#             except Exception:
#                 pass

        print >>html, "<table><thead><tr><td> </td>"
        for x in dimx.values():
            print >>html, "<th>%s</th>" % x.htmlLabel()
        print >>html, "</tr></thead>"
        for y in dimy.values():
            print >>html, "<tr><th>%s</th>" % y.htmlLabel()

            for x in dimx.values():
                print >>html, "<td>"
                idx = valsother
                idx.extend([x,y])
                test = self.getTest(idx)
                
                d="by-%s/%s/%s/%s/" % (self.prefix, self.value, y, x)
                dthumb = d+"thumb"
                try:
                    for img in dircache.listdir(dthumb):
                        print >>html, "<a href='%s/%s'><img src='%s/thumb/%s'></a>" % (urllib.quote(d), img, urllib.quote(d), img)
                except OSError:
                    print "warning: no images in %s?"%dthumb
                print >>html, "</td>"
            print >>html, "</tr>"
        print >> html, """
</table>
</body>
"""

@wvtest                
def Two_Dimensions_in_Tests():
    t = Tests(None, DimensionTest(), DimensionKern())
    WVPASSEQ([(a.__class__.__name__, b.__class__.__name__) for a, b in t.iterDimensionPairs()],
             [('DimensionTest', 'DimensionKern'), ('DimensionKern', 'DimensionTest')])
@wvtest                
def Three_Dimensions_in_Tests():
    dt = DimensionTest()
    dk = DimensionKern()
    dd = Dimension('tmp', "Tmp")
    t = Tests(None, dt, dk, dd)
    pairs = [(a.__class__.__name__, b.__class__.__name__) for a, b in t.iterDimensionPairs()]
    WVPASSEQ(pairs[0], ('DimensionTest', 'DimensionKern'))
    WVPASSEQ(pairs[1], ('DimensionKern', 'DimensionTest'))
    WVPASSEQ(pairs[2], ('DimensionTest', 'Dimension'))
    WVPASSEQ(pairs[3], ('Dimension', 'DimensionTest'))
    WVPASSEQ(pairs[4], ('DimensionKern', 'Dimension'))
    WVPASSEQ(pairs[5], ('Dimension', 'DimensionKern'))
    WVPASSEQ(len(pairs), 6)
    WVEXCEPT(KeyError, t.addTest, None, [1, 2])
    WVEXCEPT(KeyError, t.addTest, None, [1, 2, 3, 4])

    class MyTest(Test):
        pass

    t.addTest(MyTest, [1, 'a', 'A']);

    WVPASSEQ(dt.keys(), [1])
    WVPASSEQ(dk.keys(), ['a'])
    WVPASSEQ(dd.keys(), ['A'])

    v1 = dt[1]
    v2 = dk['a']
    v3 = dd['A']

    WVPASSEQ(t.getTest((v1, v2, v3)), MyTest)
    WVPASSEQ(t.getTest((v1, v3, v2)), MyTest)
    WVPASSEQ(t.getTest((v3, v2, v1)), MyTest)
    WVEXCEPT(KeyError, t.getTest, (v1, v1, v1))
    WVEXCEPT(KeyError, t.getTest, (v1, v2))
    WVEXCEPT(TypeError, t.getTest, v1)

    WVPASSEQ([d.__class__.__name__ for d in t.iterRemainingDimensions([dt])], ['DimensionKern', 'Dimension']) 
    WVPASSEQ([d.__class__.__name__ for d in t.iterRemainingDimensions([dt, dd])], ['DimensionKern'])

    WVPASSEQ(str([v for v in iterDimValues([dt, dk])]), str([[1,'a']]))
    dt.addValue(2)
    dk.addValue('b')
    WVPASSEQ(str([v for v in iterDimValues([dt, dk])]), str([[1, 'a'], [2, 'a'], [1, 'b'], [2, 'b']]))


# tests = Tests("by-kern/", DimensionKern(), DimensionTest())
    
# sys.exit(0)



# class Page:
#     def __init__(self, tests, yaxis, xaxis=None):

# class PageKern(Page):
#     prefix = 'kern'
#     name = 'kernel'
#     def __init__(self, value, xvals, yvals):
#         Page.__init__(self, value, xvals, yvals)
#     def getPreambule(self):
#         i=self.value.find(":")
#         if i>0: kver=self.value[:i]
#         else: kver=self.value
#         return "<p><a href='config-%s'>Kernel config</a></p>"%kver

# class PageClck(Page):
#     prefix = 'clck'
#     name = 'CPU clock'
#     def __init__(self, value, xvals, yvals):
#         Page.__init__(self, value, xvals, yvals)

# class PageTest(Page):
#     prefix = 'test'
#     name = 'test'
#     def __init__(self, value, xvals, yvals):
#         Page.__init__(self, value, xvals, yvals)
#     def getPreambule(self):
#         return "<p><a href='%s.sh.html'>Test source</a></p>"%(urllib.quote(self.value))



# pagesets = [ PageSet(PageKern, kernels, clocks, tests),
#              PageSet(PageTest, tests, clocks, kernels)]

# for ps in pagesets:
#     for p in ps.getPages():
#         p.generate(pagesets)

# try:
#     os.remove("index.html")
# except OSError: pass

# os.symlink("%s-%s.html"%(clocks.type, clocks[0]), "index.html")

# os.system("source-highlight -d --output-dir=. ../*.sh")
    

# for v in kernels.versions():
#     os.system("cp /boot/config-%s ."%v)
